@@ -17,7 +17,7 @@ test_that(
       3*SchurPol(n, c(2, 1, 1)) + SchurPol(n, c(1, 1, 1, 1))
     Q <- (mvp("x_1", 1, 1) + mvp("x_2", 1, 1) + mvp("x_3", 1, 1) +
             mvp("x_4", 1, 1))^4
-    expect_true(gmpoly::gmpoly2mvp(P) == Q)
+    expect_true(as_mvp_qspray(P) == Q)
   }
 )
 
@@ -34,14 +34,14 @@ test_that(
     # polynomial
     n <- 2
     lambda <- c(3,2,1)
-    expect_true(SchurPol(n, lambda) == gmpoly::gmpolyConstant(n, 0L))
+    expect_true(SchurPol(n, lambda) == as.qspray(0))
     expect_identical(SchurPol(n, lambda, algorithm = "naive"),
-                     mvp::constant(0))
+                     as.qspray(0))
     expect_identical(SchurPol(n, lambda, exact = FALSE, algorithm = "naive"),
                      mvp::constant(0))
     expect_identical(SchurPol(n, lambda, algorithm = "naive",
                              basis = "MSF"),
-                     mvp::constant(0))
+                     as.qspray(0))
     expect_identical(SchurPol(n, lambda, exact = FALSE, algorithm = "naive",
                              basis = "MSF"),
                      mvp::constant(0))
@@ -79,27 +79,10 @@ test_that(
 
 test_that(
   "SchurPol is correct", {
-    bigqMonomial <- function(vars, powers){
-      do.call(prod, mapply(gmp::pow.bigq, vars, powers, SIMPLIFY = FALSE))
-    }
-    evalPol <- function(pol, x){
-      vars <- paste0("x_", seq_along(x))
-      polValue <- pol
-      for(i in 1L:length(x)){
-        polValue <- mvp::subsmvp(polValue, vars[i], mvp::mvp(x[i],1,1))
-      }
-      polValue$names <- lapply(polValue$names, as.bigq)
-      terms <-
-        mapply(bigqMonomial, polValue$names, polValue$power, SIMPLIFY = FALSE)
-      value <-
-        do.call(sum, mapply("*", polValue$coeffs, terms, SIMPLIFY = FALSE))
-      value
-    }
-    #
     lambda <- c(3,2)
     pol <- SchurPol(4, lambda, algorithm = "naive")
-    x <- as.character(as.bigq(c(6L,-7L,8L,9L), c(1L,2L,3L,4L)))
-    polEval <- evalPol(pol, x)
+    x <- as.bigq(c(6L,-7L,8L,9L), c(1L,2L,3L,4L))
+    polEval <- qspray::evalQspray(pol, x)
     expect_identical(polEval, Schur(as.bigq(x), lambda))
   }
 )
@@ -110,7 +93,7 @@ test_that(
     P1 <- SchurPol(n, c(3, 2)) + 2 * SchurPol(n, c(2, 2, 1)) +
       SchurPol(n, c(3, 1, 1)) + 2 * SchurPol(n, c(2, 1, 1, 1)) +
       SchurPol(n, c(1, 1, 1, 1, 1))
-    P2 <- ESFpoly(n, c(2, 2, 1))
-    expect_true(gmpoly::gmpoly2mvp(P1) == P2)
+    P2 <- qspray::ESFpoly(n, c(2, 2, 1))
+    expect_true(P1 == P2)
   }
 )
